require 'spec_helper'

describe RocketGate::Hashable do
  describe '.hashable' do
    let(:test_class) { Class.new }

    it 'records a hash of display names and their attribute mappings' do
      expect(test_class).to receive(:hashable).with({:One => :one, "two" => :two})
      test_class.class_eval do
        include RocketGate::Hashable
        attr_accessor :one, :two, :three
        hashable({:One => :one, "two" => :two})
      end
    end

    it 'records a list of mergable items appearing after the mappings' do
      expect(test_class).to receive(:hashable).with({one: :one}, :two, :three)
      test_class.class_eval do
        include RocketGate::Hashable
        attr_accessor :one, :two, :three
        hashable({one: :one}, :two, :three)
      end
    end

    it 'allows display names to be mapped to procs' do
      expect(test_class).to receive(:hashable).with({one: an_instance_of(Proc)})
      expect do
        test_class.class_eval do
          include RocketGate::Hashable
          hashable({one: Proc.new { true } })
        end
      end.not_to raise_error
    end

    it 'allows mergable items to procs' do
      expect(test_class).to receive(:hashable).with({one: :one}, an_instance_of(Proc))
      expect do
        test_class.class_eval do
          include RocketGate::Hashable
          attr_accessor :one
          hashable({one: :one}, Proc.new { true })
        end
      end.not_to raise_error
    end

    it 'complains about display names mapped to attributes that are not real' do
      expect do
        test_class.class_eval do
          include RocketGate::Hashable
          attr_accessor :one, :two
          hashable({:One => :one, "two" => :two, "3" => :three})
        end
      end.to raise_error(RocketGate::ValidationError, 'Invalid attribute: three')
    end

    it 'complains about mergable items which are not real' do
      expect do
        test_class.class_eval do
          include RocketGate::Hashable
          attr_accessor :one, :two
          hashable({one: :one}, :two, :three)
        end
      end.to raise_error(RocketGate::ValidationError, 'Invalid attribute: three')
    end
  end

  describe '#to_hash' do
    let(:test_class) do
      c = Class.new
      c.class_eval do
        include RocketGate::Hashable
        attr_accessor :one, :two
        define_method :three do
          { "3" => 1+2 }
        end
        hashable({one: :one, 'Two' => :two}, :three, ->{ { four: 4 } })
      end
      c.new
    end

    it 'builds a hash using the mappings and mergables provided' do
      expect(test_class.to_hash).to eq({one: nil, 'Two' => nil, '3' => 3, four: 4})
    end
  end
end
