require 'spec_helper'

describe 'RocketGate Integration Certification Test Cases', integration: true do
  before do
    RocketGate.configuration.testing!
    RocketGate.configuration.require_scrub = false
  end

  let(:request) { RocketGate::Request.new(transaction) }
  let(:base_tx) do
    RocketGate::Transaction.new.tap do |tx|
      tx.id                 = 'b7bb0d51-ce1a-4dd2-b0f4-61e5ff5449db'
      tx.amount             = 1.00

      tx.customer = RocketGate::Customer.new.tap do |cust|
        cust.id             = '17da86fd-7bac-4fa0-88c1-7396b3897ac2'
        cust.first_name     = 'John'
        cust.last_name      = 'Test'
        cust.street_address = '1234 Main Street'
        cust.city           = 'Las Vegas'
        cust.state          = 'NV'
        cust.postal_code    = '89141'
        cust.country        = 'US'
        cust.email          = 'user@test.com'
        cust.ip_address     = '216.239.116.167'
      end

      tx.credit_card = RocketGate::CreditCard.new.tap do |cc|
        cc.number           = '4012 8888 8888 1881'
        cc.exp_month        = '08'
        cc.exp_year         = '2042'
        cc.cvv              = '111'
      end
    end
  end

  describe 'Test Case #1 (auth-only, then ticket for authorization by ID)' do
    let(:transaction) { base_tx.dup }
    before { transaction.type = RocketGate::Transaction::TYPE[:auth] }

    it 'successfully authorizes, confirms, and settles a transaction' do
      auth_response = RocketGate.send_request!(RocketGate::Request.new(transaction))
      expect(auth_response).to be_success
      expect(auth_response.authorization).not_to be_declined

      confirmation_response = auth_response.confirm!
      expect(confirmation_response).to be_success

      ticket_tx = RocketGate::Transaction.new.tap do |tx|
        tx.reference_id = auth_response.authorization.reference_id
        tx.type = RocketGate::Transaction::TYPE[:ticket]
      end

      ticket_response = RocketGate.send_request!(RocketGate::Request.new(ticket_tx))
      expect(ticket_response).to be_success
      expect(ticket_response).to be_authorized
      expect(ticket_response.authorization).not_to be_declined
      expect(ticket_response.authorization.approved_amount).to eq 1.0
      expect(ticket_response.authorization.approved_currency).to eq 'USD'
      expect(ticket_response.authorization.card_last_four).to eq '1881'
      expect(ticket_response.authorization.card_expiration).to eq '0842'
      expect(ticket_response.authorization.card_hash).to eq '3RPN+a+d07r0bOlq7NcWPKvzgcyyHmPxXw+hCxxmP6k='
      expect(ticket_response.authorization.card_type).to eq :credit
      expect(ticket_response.authorization.issuer_network).to eq :visa
      expect(ticket_response.authorization.reference_id).not_to be_nil
    end
  end

  describe 'Test Case #2 (make sale, then void sale by ID)' do
    let(:transaction) { base_tx.dup }
    before { transaction.type = RocketGate::Transaction::TYPE[:purchase] }

    it 'successfully charges and voids a transaction' do
      sale_response = RocketGate.send_request!(RocketGate::Request.new(transaction))
      expect(sale_response).to be_success
      expect(sale_response.authorization).not_to be_declined

      confirmation_response = sale_response.confirm!
      expect(confirmation_response).to be_success

      void_tx = RocketGate::Transaction.new.tap do |tx|
        tx.reference_id = sale_response.authorization.reference_id
        tx.type = RocketGate::Transaction::TYPE[:void]
      end
      void_response = RocketGate.send_request!(RocketGate::Request.new(void_tx))

      expect(void_response).to be_success
      expect(void_response).to be_authorized
      expect(void_response.authorization).not_to be_declined
      expect(void_response.authorization.approved_amount).to eq 1.0
      expect(void_response.authorization.approved_currency).to eq 'USD'
      expect(void_response.authorization.card_last_four).to eq '1881'
      expect(void_response.authorization.card_expiration).to eq '0842'
      expect(void_response.authorization.card_hash).to eq '3RPN+a+d07r0bOlq7NcWPKvzgcyyHmPxXw+hCxxmP6k='
    end
  end

  describe 'Test Case #3 (failed AVS check, response of N)' do
    let(:transaction) { base_tx.dup }
    before { transaction.customer.postal_code = '00008' }

    it 'returns an unsuccessful response' do
      response = RocketGate.send_request!(RocketGate::Request.new(transaction))
      expect(response).not_to be_success
      expect(response.authorization).to be_declined
      expect(response.authorization.avs_ok?).to be false
    end
  end

  describe 'Test Case #4 (failed CVV check, response of N)' do
    let(:transaction) { base_tx.dup }
    before { transaction.credit_card.cvv = '0001' }

    it 'returns an unsuccessful response' do
      response = RocketGate.send_request!(RocketGate::Request.new(transaction))
      expect(response).not_to be_success
      expect(response.authorization).to be_declined
      expect(response.authorization.cvv_ok?).to be false
    end
  end

  describe 'Test Case #5 (declined, card over limit)' do
    let(:transaction) { base_tx.dup }
    before { transaction.amount = 0.02 }

    it 'returns an unsuccessful response' do
      response = RocketGate.send_request!(RocketGate::Request.new(transaction))
      expect(response).not_to be_success
      expect(response.authorization).to be_declined
      expect(response.authorization.reason_code).to eq :declined_over_limit
    end
  end
end
