require 'spec_helper'

describe RocketGate::Request do
  describe '.new' do
    subject { RocketGate::Request.new(transaction) }

    let(:transaction) { double(:transaction) }

    it 'sets the transaction' do
      expect(subject.transaction).to eq transaction
    end
  end

  describe '#base_parameters' do
    subject { request.base_parameters }
    let(:request) { RocketGate::Request.new }

    before { config.testing! }

    let(:config) { RocketGate.configuration }

    it 'returns a hash that respects configuration' do
      request.transaction = double(:transaction, is_referenced?: false, is_hashed?: false)
      expect(subject[:version]).to eq 'R1.2'
      expect(subject[:merchantID]).to eq '1'
      expect(subject[:merchantPassword]).to eq 'testpassword'
      expect(subject[:avsCheck]).to eq 'YES'
      expect(subject[:cvv2Check]).to eq 'YES'
      expect(subject[:scrub]).to eq 'YES'
    end

    context 'when the transaction is referenced' do
      before { request.transaction = double(:transaction, is_referenced?: true, is_hashed?: true) }

      it 'does not include AVS, CVV or Scrub configuration' do
        expect(subject[:version]).to eq 'R1.2'
        expect(subject[:merchantID]).to eq '1'
        expect(subject[:merchantPassword]).to eq 'testpassword'
      end
    end
  end

  describe '#build_gateway_request' do
    subject { RocketGate::Request.new(transaction).build_gateway_request }
    let(:transaction) do
      RocketGate::Transaction.new.tap do |tx|
        tx.amount = 10.97
        tx.currency = 'GBP'
        tx.id = 'abbb2170-6a71-4d96-aa8f-ceb8b7d1a7ca'

        tx.customer = RocketGate::Customer.new.tap do |cust|
          cust.id = 'Customer-1'
          cust.first_name = 'Test'
          cust.last_name = 'Person'
          cust.street_address = '123 Fake Street'
          cust.city = 'Beverly Hills'
          cust.state = 'CA'
          cust.postal_code = '90210'
          cust.country = 'US'
          cust.email = 'test@example.com'
          cust.ip_address = '127.0.0.1'
        end

        tx.credit_card = RocketGate::CreditCard.new.tap do |cc|
          cc.number = '4111111111111111'
          cc.exp_month = '02'
          cc.exp_year = '2015'
          cc.cvv = '999'
        end
      end
    end

    before do
      RocketGate.configuration.testing!
      RocketGate.configuration.require_avs = false
    end

    it 'matches the expected xml output' do
      expected = %Q{<?xml version="1.0" encoding="UTF-8"?><gatewayRequest><version>R1.2</version><merchantID>1</merchantID><merchantPassword>testpassword</merchantPassword><cvv2Check>YES</cvv2Check><avsCheck>IGNORE</avsCheck><scrub>YES</scrub><amount>10.97</amount><currency>GBP</currency><merchantInvoiceID>abbb2170-6a71-4d96-aa8f-ceb8b7d1a7ca</merchantInvoiceID><transactionType>CC_AUTH</transactionType><cardNo>4111111111111111</cardNo><expireMonth>02</expireMonth><expireYear>2015</expireYear><cvv2>999</cvv2><merchantCustomerID>Customer-1</merchantCustomerID><customerFirstName>Test</customerFirstName><customerLastName>Person</customerLastName><billingAddress>123 Fake Street</billingAddress><billingCity>Beverly Hills</billingCity><billingState>CA</billingState><billingZipCode>90210</billingZipCode><billingCountry>US</billingCountry><email>test@example.com</email><ipAddress>127.0.0.1</ipAddress></gatewayRequest>}
      expect(subject).to eq expected
    end
  end

  describe '#yes_or_ignore' do
    it 'returns yes for true' do
      expect(RocketGate::Request.new.yes_or_ignore(true)).to eq 'YES'
    end

    it 'returns ignore for false' do
      expect(RocketGate::Request.new.yes_or_ignore(false)).to eq 'IGNORE'
    end
  end
end
