require 'spec_helper'

describe RocketGate::Response do
  before { RocketGate.configuration.testing! }

  let(:xml) { '<?xml version="1.0" encoding="UTF-8"?><gatewayResponse><cardType>VISA</cardType><approvedAmount>9.99</approvedAmount><retrievalNo>1000148e87eab19</retrievalNo><approvedCurrency>USD</approvedCurrency><reasonCode>0</reasonCode><merchantAccount>1</merchantAccount><version>1.0</version><cardLastFour>1111</cardLastFour><cardHash>m77xlHZiPKVsF9p1/VdzTb+CUwaGBDpuSRxtcb7+j24=</cardHash><guidNo>1000148E87EAB19</guidNo><responseCode>0</responseCode><authNo>892343</authNo><cardExpiration>0210</cardExpiration><payType>CREDIT</payType></gatewayResponse>' }

  describe '.from_xml' do
    subject { RocketGate::Response.from_xml(xml) }
    let(:response) { RocketGate::Response.new }

    it 'builds a new instance with the xml document' do
      expect(RocketGate::Response).to receive(:new).and_return(response)
      expect(Nokogiri).to receive(:parse).with(xml).and_call_original
      expect(response).to receive(:xml_document=).with(an_instance_of(Nokogiri::XML::Document))
      expect(response).to receive(:build!)
      subject
    end
  end

  describe '#build!' do
    subject { response.build! }
    let(:response) { RocketGate::Response.new.tap{|r| r.xml_document = Nokogiri.parse(xml) } }
    let(:authorization) { RocketGate::Authorization.new }

    before { allow(RocketGate::Authorization).to receive(:new).and_return(authorization) }

    it 'fills in the response code' do
      expect{subject}.to change{response.response_code}.from(nil).to(:success)
    end

    it 'builds an authorization' do
      expect{subject}.to change{response.authorization}
        .from(nil).to(authorization)

      expect(authorization.approved_amount).to eq 9.99
      expect(authorization.approved_currency).to eq 'USD'
      expect(authorization.auth_code).to eq '892343'
      expect(authorization.avs_response).to be_nil
      expect(authorization.card_expiration).to eq '0210'
      expect(authorization.card_hash).to eq 'm77xlHZiPKVsF9p1/VdzTb+CUwaGBDpuSRxtcb7+j24='
      expect(authorization.card_last_four).to eq '1111'
      expect(authorization.cvv_response).to be_nil
      expect(authorization.reason_code).to eq :success
      expect(authorization.reference_id).to eq '1000148E87EAB19'
    end
  end

  describe '#confirm!' do
    subject { response.confirm! }
    let(:response) { RocketGate::Response.from_xml(xml) }

    before do
      allow(response).to receive(:success?).and_return(success)
      allow(RocketGate).to receive(:send_request!).and_return(response)
    end

    context 'when the request was successful' do
      let(:success) { true }
      let(:transaction) { RocketGate::Transaction.new }

      before { allow(RocketGate::Transaction).to receive(:new).and_return(transaction) }

      it 'builds a confirmation transaction' do
        expect(RocketGate::Transaction).to receive(:new)
        expect(transaction).to receive(:reference_id=)
          .with('1000148E87EAB19').and_call_original
        expect(transaction).to receive(:type=)
          .with(RocketGate::Transaction::TYPE[:confirmation]).and_call_original
        expect(RocketGate::Request).to receive(:new)
          .with(transaction).and_call_original

        subject
      end

      it 'sends the confirmation transaction' do
        expect(RocketGate).to receive(:send_request!).with(an_instance_of(RocketGate::Request))
        subject
      end

      it 'returns the confirmation response' do
        allow(RocketGate).to receive(:send_request!).and_return(response)
        expect(subject).to eq response
      end
    end

    context 'when the request was not successful' do
      let(:success) { false }

      it 'raises an error' do
        expect{subject}.to raise_error(RocketGate::AuthorizationError, 'Unable to confirm unauthorized transaction')
      end
    end
  end

  describe '#authorized?' do
    subject { response.authorized? }
    let(:response) { RocketGate::Response.new }

    context 'when there is a successful authorization' do
      before { response.authorization = double(:authorization, success?: true) }

      it 'returns true' do
        expect(subject).to be true
      end
    end

    context 'when there is an unsuccessful authorization' do
      before { response.authorization = double(:authorization, success?: false) }

      it 'returns true' do
        expect(subject).to be false
      end
    end

    context 'when there is no authorization' do
      before { response.authorization = nil }

      it 'returns true' do
        expect(subject).to be false
      end
    end
  end

  describe '#success?' do
    subject { response.success? }
    let(:response) { RocketGate::Response.new }

    before do
      allow(response).to receive(:authorized?).and_return(authorized)
      response.response_code = response_code
    end

    context 'when the request was authorized' do
      let(:authorized) { true }
      let(:response_code) { :success }

      it 'returns true' do
        expect(subject).to be true
      end
    end

    context 'when the request was not authorized' do
      let(:authorized) { false }
      let(:response_code) { :success }

      it 'returns false' do
        expect(subject).to be false
      end
    end

    context 'when the request was not successful' do
      let(:authorized) { true }
      let(:response_code) { :bank_decline }

      it 'returns false' do
        expect(subject).to be false
      end
    end
  end

  describe '#find_value' do
    subject { response.send(:find_value, key) }
    let(:response) { RocketGate::Response.from_xml(xml) }

    context 'when the key is present' do
      let(:key) { 'authNo' }

      it 'returns the corresponding value' do
        expect(subject).to eq '892343'
      end
    end

    context 'when the key is not present' do
      let(:key) { 'missingKey' }

      it 'returns nothing' do
        expect(subject).to be_nil
      end
    end

    context 'when the response is not what we expect' do
      let(:xml) { '<?xml version="1.0" encoding="UTF-8"?><unexpectedDocument />' }

      it 'raises an error' do
        expect{subject}.to raise_error(RocketGate::GatewayResponseError, 'Unexpected response format')
      end
    end
  end
end
