require 'spec_helper'

describe RocketGate do
  describe '.configuration' do
    subject { RocketGate.configuration }

    context 'when configuration is available' do
      before { RocketGate.configure {|c| c.merchant_id = '123' } }

      it 'returns it' do
        expect(RocketGate::Configuration).not_to receive(:new)
        expect(subject.merchant_id).to eq '123'
      end
    end

    context 'when configuration is not available' do
      before { RocketGate.reset! }

      it 'supplies a default config' do
        expect(subject.request_host).to eq RocketGate::Configuration::GATEWAY_LIVE_HOST
      end
    end
  end

  describe '.configure' do
    subject { RocketGate.configure {|c| c.require_avs = false } }
    let(:config) { RocketGate.configuration }

    it 'applies settings from a config block' do
      expect{subject}.to change{config.require_avs}.from(true).to(false)
    end
  end

  describe '.connection' do
    subject { RocketGate.connection }
    let(:config) { RocketGate.configuration }

    before { config.testing! }

    it 'validates the configuration' do
      expect(config).to receive(:validate!)
      subject
    end

    it 'returns a connection' do
      expect(RocketGate::Connection).to receive(:new)
        .with(config.request_host, config.request_port, config.request_path)
        .and_call_original
      expect(subject).to be_an_instance_of(RocketGate::Connection)
    end
  end

  describe '.reset!' do
    subject { RocketGate.reset! }
    before do
      RocketGate.configuration.testing!
      RocketGate.connection
    end

    it 'clears the memoized configuration' do
      expect(RocketGate.instance_variable_get(:@configuration)).not_to be_nil
      subject
      expect(RocketGate.instance_variable_get(:@configuration)).to be_nil
    end

    it 'clears the memoized connection' do
      expect(RocketGate.instance_variable_get(:@connection)).not_to be_nil
      subject
      expect(RocketGate.instance_variable_get(:@connection)).to be_nil
    end
  end

  describe '.send_request!' do
    subject { RocketGate.send_request!(request) }

    before do
      RocketGate.configuration.testing!
      allow(RocketGate.connection).to receive(:make_request!).and_return(response)
    end

    let(:request) { double(:request, build_gateway_request: 'request') }
    let(:response) { double(:response, code: 200, body: xml) }
    let(:xml) { '<?xml version="1.0" encoding="UTF-8"?><gatewayResponse><cardType>VISA</cardType><approvedAmount>9.99</approvedAmount><retrievalNo>1000148e87eab19</retrievalNo><approvedCurrency>USD</approvedCurrency><reasonCode>0</reasonCode><merchantAccount>1</merchantAccount><version>1.0</version><cardLastFour>1111</cardLastFour><cardHash>m77xlHZiPKVsF9p1/VdzTb+CUwaGBDpuSRxtcb7+j24=</cardHash><guidNo>1000148E87EAB19</guidNo><responseCode>0</responseCode><authNo>892343</authNo><cardExpiration>0210</cardExpiration><payType>CREDIT</payType></gatewayResponse>' }

    it 'sends the request via the connection' do
      expect(RocketGate.connection).to receive(:make_request!).with('request')
      subject
    end

    it 'builds a response object from the parsed body' do
      expect(RocketGate::Response).to receive(:from_xml).with(xml).and_call_original

      subject
    end

    it 'does not raise an error' do
      expect{subject}.not_to raise_error
    end

    context 'when the request was unsuccessful' do
      before { allow(response).to receive(:code).and_return(500) }

      it 'raises an error' do
        expect{subject}.to raise_error(RocketGate::GatewayRequestError, 'Unexpected status code in gateway response: 500')
      end
    end
  end
end
