require 'spec_helper'

describe RocketGate::Transaction do
  describe '.new' do
    let(:customer) { double(:customer) }
    let(:credit_card) { double(:credit_card) }

    it 'accepts an arbitrary set of arguments and assigns them to certain attributes' do
      transaction = RocketGate::Transaction.new(12.34, customer, credit_card, 'GBP')
      expect(transaction.amount).to eq 12.34
      expect(transaction.customer).to eq customer
      expect(transaction.credit_card).to eq credit_card
      expect(transaction.currency).to eq 'GBP'
    end
  end

  describe '#credit_card_hash' do
    subject { transaction.credit_card_hash }
    let(:transaction) do
      RocketGate::Transaction.new.tap {|tx| tx.credit_card = credit_card }
    end
    let(:credit_card) { double(:credit_card) }

    it 'delegates to the credit card' do
      expect(credit_card).to receive(:to_hash)
      subject
    end
  end

  describe '#customer_hash' do
    subject { transaction.customer_hash }
    let(:transaction) do
      RocketGate::Transaction.new.tap {|tx| tx.customer = customer }
    end
    let(:customer) { double(:customer) }

    it 'delegates to the customer' do
      expect(customer).to receive(:to_hash)
      subject
    end
  end

  describe '#id' do
    let(:transaction) { RocketGate::Transaction.new }

    it 'returns the supplied ID when set' do
      transaction.id = 'abc123'
      expect(transaction.id).to eq 'abc123'
    end

    it 'generates a random UUID when not set' do
      expect(SecureRandom).to receive(:uuid).and_return('def456')
      expect(transaction.id).to eq 'def456'
    end
  end

  describe '#is_referenced?' do
    let(:transaction) do
      RocketGate::Transaction.new.tap do |tx|
        tx.reference_id = 'abc123'
        tx.type         = type
      end
    end

    context 'when it is a ticket transaction' do
      let(:type) { RocketGate::Transaction::TYPE[:ticket] }

      it 'returns true' do
        expect(transaction.is_referenced?).to eq true
      end
    end

    context 'when it is a void transaction' do
      let(:type) { RocketGate::Transaction::TYPE[:void] }

      it 'returns true' do
        expect(transaction.is_referenced?).to eq true
      end
    end

    context 'when it is not a referenced transaction' do
      let(:type) { RocketGate::Transaction::TYPE[:auth] }

      it 'returns true' do
        expect(transaction.is_referenced?).to eq false
      end
    end
  end

  describe '#type' do
    let(:transaction) { RocketGate::Transaction.new }

    it 'returns auth type by default' do
      expect(transaction.type).to eq RocketGate::Transaction::TYPE[:auth]
    end

    it 'returns the assigned value when set' do
      transaction.type = RocketGate::Transaction::TYPE[:purchase]
      expect(transaction.type).to eq RocketGate::Transaction::TYPE[:purchase]
    end
  end

  describe '#type=' do
    let(:transaction) { RocketGate::Transaction.new }

    it 'converts symbols to their associated string values' do
      transaction.type = :auth
      expect(transaction.type).to eq RocketGate::Transaction::TYPE[:auth]
    end

    it 'raises an error if an unknown type is specified' do
      expect{ transaction.type = :fake_type }.to raise_error(RocketGate::ValidationError, 'Unknown transaction type: fake_type')
      expect{ transaction.type = 'FAKETYPE' }.to raise_error(RocketGate::ValidationError, 'Unknown transaction type: FAKETYPE')
    end
  end

  describe '#to_hash' do
    context 'when it is a standard transaction' do
      let(:transaction) do
        RocketGate::Transaction.new.tap do |tx|
          tx.id                 = '90c3e1d8-14f0-4aea-9eda-a7a59396e264'
          tx.amount             = 12.34
          tx.type               = RocketGate::Transaction::TYPE[:auth]

          tx.customer = RocketGate::Customer.new.tap do |cust|
            cust.id             = '07f7b231-9b29-46d4-9c19-cd6790e712b5'
            cust.first_name     = 'Test'
            cust.last_name      = 'Person'
            cust.street_address = '123 Fake Street'
            cust.city           = 'Beverly Hills'
            cust.state          = 'CA'
            cust.postal_code    = '90210'
            cust.country        = 'US'
            cust.email          = 'test@example.com'
            cust.username       = 'tester'
            cust.ip_address     = '127.0.0.1'
          end

          tx.credit_card = RocketGate::CreditCard.new.tap do |cc|
            cc.number           = '4111111111111111'
            cc.exp_month        = '01'
            cc.exp_year         = '2020'
            cc.cvv              = '999'
          end
        end
      end

      let(:expected) do
        {
          cardNo: '4111111111111111',
          expireMonth: '01',
          expireYear: '2020',
          cvv2: '999',
          amount: 12.34,
          currency: 'USD',
          merchantCustomerID: '07f7b231-9b29-46d4-9c19-cd6790e712b5',
          merchantInvoiceID: '90c3e1d8-14f0-4aea-9eda-a7a59396e264',
          customerFirstName: 'Test',
          customerLastName: 'Person',
          billingAddress: '123 Fake Street',
          billingCity: 'Beverly Hills',
          billingState: 'CA',
          billingZipCode: '90210',
          billingCountry: 'US',
          email: 'test@example.com',
          username: 'tester',
          ipAddress: '127.0.0.1',
          referenceGUID: nil,
          transactionType: 'CC_AUTH'
        }
      end

      it 'returns a hash of the transaction attributes' do
        expect(transaction.to_hash).to eq expected
      end
    end

    context 'when it is a ticket transaction' do
      let(:transaction) do
        RocketGate::Transaction.new.tap do |tx|
          tx.reference_id = 'abc123'
          tx.type         = RocketGate::Transaction::TYPE[:ticket]
        end
      end

      let(:expected) do
        { referenceGUID: 'abc123', transactionType: 'CC_TICKET' }
      end

      it 'returns only the reference id and type' do
        expect(transaction.to_hash).to eq expected
      end
    end

    context 'when it is a void transaction' do
      let(:transaction) do
        RocketGate::Transaction.new.tap do |tx|
          tx.reference_id = 'abc123'
          tx.type         = RocketGate::Transaction::TYPE[:void]
        end
      end

      let(:expected) do
        { referenceGUID: 'abc123', transactionType: 'CC_VOID' }
      end

      it 'returns only the reference id and type' do
        expect(transaction.to_hash).to eq expected
      end
    end

    context 'when it is a confirmation' do
      let(:transaction) do
        RocketGate::Transaction.new.tap do |tx|
          tx.reference_id = 'abc123'
          tx.type         = RocketGate::Transaction::TYPE[:confirmation]
        end
      end

      let(:expected) do
        { referenceGUID: 'abc123', transactionType: 'CC_CONFIRM' }
      end

      it 'returns only the reference id and type' do
        expect(transaction.to_hash).to eq expected
      end
    end
  end

  describe '#valid?' do
    subject { transaction.valid? }
    context 'when it is a referenced transaction' do
      let(:transaction) { RocketGate::Transaction.new }
      before { allow(transaction).to receive(:is_referenced?).and_return(true) }

      it 'bypasses normal validation rules' do
        expect(subject).to be true
      end
    end

    context 'when it is a standard transaction' do
      let(:transaction) { RocketGate::Transaction.new }
      before { allow(transaction).to receive(:is_referenced?).and_return(false) }

      it 'defers to normal validation rules' do
        expect(subject).to be false
      end
    end
  end
end
